﻿#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=1		// Use modern global access method and strict wave access.

function calcInitialGuess(dNdlogmrBC,m_star)
	wave dNdlogmrBC, m_star
	//Calculate the initial guess by dividing dNdlogmrBC by charge fraction and the resolution at the CPMA setpoint
	//Then, zero values will be added to bins which were added due to diffusion or multiple charging
	
	//We will need to calculate the charge fraction and resolution at each CPMA setpoint (i.e. m_star)
	
	make/O/N=(numpnts(m_star)) dp
	make/O/N=(numpnts(m_star)) fn
	make/O/N=(dimsize(dNdlogmrBC,0),dimsize(dNdlogmrBC,1)) ini_guess_square
	
	duplicate/O dNdlogmrBC ini_guess
	variable ii //counting variable
	nvar C //assumed prefactor
	nvar Dm //assumed mass-mobility exponent
	nvar Temperature
	
	//need to make a wave to take the rBC mass distribution of the ith CPMA setpoint
	make/O/N=(dimsize(dNdlogmrBC,0)) mStarBcDistribution
	for(ii=0;ii<numpnts(m_star)+1;ii+=1)
		dp[ii] = (m_star[ii]/C)^(1/Dm) //mpbility diameter in meters
		fn[ii] = eq_charge_frac(dp[ii],1,Temperature) //calculate the charge fraction
		mStarBcDistribution = dNdlogmrBC[p][ii] 
		print mStarBcDistribution
		print dp[ii]
		print fn[ii]
		//check this tomorrow - November 9th
		ini_guess_square[][ii] = dNdlogmrBC[p][ii]/(dp[ii]*fn[ii]) //calculate the square initial guess, like seen in Stolzenberg, 2008
	endfor
	
	// now need to add zeros to bins which are "non" real (i.e. we added bins to accomodate diffusion and multiple charging)
	// This was done in the Kernal function
	nvar Temperature
	nvar bins_added_on_left, bins_added_on_right
	make/O/N=(dimsize(ini_guess_square,0),(dimsize(ini_guess_square,1)+bins_added_on_left+bins_added_on_right)) ini_guess
	//ini_guess[][] = 0 //make it all zeros at first
	//ini_guess[][bins_added_on_left,(dimsize(ini_guess_square,1)+bins_added_on_left)] = ini_guess_square
	
	//for (ii=0;ii<numpnts(m_star)+1;ii+=1)
	   
	   
		
	//endfor
		
end


function Twomey_inversion_3DV1(Response,GammaK,max_iterations,midpoints,m_bin,m_upper,m_lower_midpoints,ini_guess)
	//%%%
	//% INPUT
	//% R:     is the response of the instrument (i.e. dN/dlogm_rBC)
	//% Gamma: is the kernel of the instrument
	//% ini_guess:  initial guess as dN2_dlog_mp_rBC
	//% max_iterations: sets the maximum number of loops through the twomey solver
	//%
	//% OUTPUT
	//% dN2_dlog_mp_rBC: is the solution to dN2_dlog_mp_rBC * Gamma =  dN_dlogmrBC
	//% N_response_new: is a vector of the number concentration calcualated from
	//% the last iteration of the Twomey loop
	//% Response_new: is the 
	//% REFERENCES
	//% Twomey (1975), Markowski (1987), Hogan (2016)
	//%Written by Kurtis Broda, Oct 18, 2016
	//%%%
	wave response, GammaK, midpoints, m_bin, m_upper,m_lower_midpoints, ini_guess
	variable max_iterations
	variable ii,jj //for counting
	
	//this section below is obsolete
//	//see notes from October 20th ish - note transpose rules useful for solving xA=B where A is unknown and this is an over/underdetermined system
////	matrixOP/O test_A = GammaK^t
//	matrixOP/O test_B = Response^t
//	
//	//print wavetype(GammaK)
//	//print wavetype(Response)
//	
//	//MatrixLLS/M=1 test_A test_B
//	//wave M_B
//	//print M_B
//	//print V_flag
//	MatrixSolve SV, GammaK,test_B
//	wave M_X
//	MatrixOP/O ini_guess = M_X^t
//	ini_guess=ini_guess[p][q] < 0 ? 1 : ini_guess[p][q];
//	//print ini_guess
	
	//now make a matrix that sets points which cannot exist to zero
	make/O/N=(numpnts(midpoints),numpnts(m_bin)) isreal
	
	for (ii=0;ii<numpnts(m_bin);ii+=1)
		for (jj=0;jj<numpnts(midpoints);jj+=1)
			isreal[jj][ii] = m_upper[ii]-m_lower_midpoints[jj]
			if (isreal[jj][ii] <=0)
				isreal[jj][ii]=0
			endif
			if (isreal[jj][ii] >0)
				isreal[jj][ii]=1
			endif
		endfor
	endfor
	
	//edit isreal
	
	make/O/D/N=(numpnts(midpoints),numpnts(m_bin)) dN2_dlog_mp_rBC=ini_guess*isreal
	make/O/D/N=(max_iterations-1) max_error
	make/O/D/N=(max_iterations-1) N_response_new
	
	variable k=0
	variable convergence_check=1
	variable tolerance=0.5
	make/O/D/N=(numpnts(midpoints),numpnts(m_bin)) Response_new
	make/O/D/N=(dimsize(Response,0),dimsize(Response,1)) a
	make/O/D/N=(numpnts(midpoints),numpnts(m_bin)) Correction
	make/O/D/N=(numpnts(midpoints),numpnts(m_bin)) dN2_dlog_mp_rBC_new=0
	make/O/N=(numpnts(midpoints),numpnts(m_bin)) test
	
	do
		matrixOP/O Response_new = dN2_dlog_mp_rBC x GammaK^t
		a = Response/Response_new
		a=a[p][q] > 1e40 ? 1 : a[p][q]; //remove the infinities!
		a=a[p][q] < -1e40 ? 1 : a[p][q]; //remove the negative infinities!
		MatrixOP/O a = ReplaceNans(a,1) //replace the nans
		MatrixOP/O Correction=(1+(a-1) x GammaK) //calculate the correction
		
		dN2_dlog_mp_rBC_new = Correction*dN2_dlog_mp_rBC
		
		MatrixOP/O converge_check = maxval(abs(dN2_dlog_mp_rBC_new/dN2_dlog_mp_rBC))
		max_error[k] = converge_check[0]
		
		dN2_dlog_mp_rBC = dN2_dlog_mp_rBC_new;

		k+=1
	while (k<max_iterations)
	
	
	
	
	
end