#pragma rtGlobals=1		// Use modern global access method and strict wave access.

//CALCULATE M_STAR BASED ON VOLTAGE, SPEED, AND ELECTRIC CHARGE
function calc_m_star(electric_charge,V,omega)
	variable electric_charge, V, omega
	variable CPMA_gap, CPMA_r1, CPMA_r2, CPMA_rc
	CPMA_gap   =   .001;             //  gap - distance between cylinders (m)
	CPMA_r1   =   .060;              // r1 - inner radius of APM (m)
	CPMA_r2   =   CPMA_r1+CPMA_gap;             // r2 - outer radius of APM (m)
	CPMA_rc = (CPMA_r2+CPMA_r1)/2; 
	//calculate the m_star
	variable mc
	mc = electric_charge*V/omega^2/CPMA_rc^2/ln(CPMA_r2/CPMA_r1);
	return mc
end

//CALCULATE THE M_MAX BASED ON THE DIFFUSION TRANSFER FUNCTION
function calc_m_max(m_star,electric_charge,omega,V,mass_mob_pref,flowrate_CPMA,mass_mob_exp,Pressure,Temperature,omega_hat)
	variable m_star,electric_charge,omega,V,mass_mob_pref,flowrate_CPMA,mass_mob_exp,Pressure,Temperature,omega_hat
	variable m_max = m_star
	variable trans = 1
	variable B
	variable count = 1
	//step to the right to find where the CPMA transfer function is small
		do 
			m_max *=1.0025
			B = mass2mob(m_max,mass_mob_pref,mass_mob_exp,Pressure,Temperature)
			trans = CPMA_diff_model_fixedgrid(m_max,B ,electric_charge,flowrate_CPMA,omega,V,Temperature,omega_hat)
			if (count>1000)
				 break
			endif
			count +=1
			//print trans
		while (trans>0.0025)
		//print count
	return m_max

end

//CALCULATE Rm BASED ON M_MAX AND M_STAR
function Rm_calc(m_star,m_max)
	variable m_star,m_max
	variable Rm
	
	Rm = 1/(m_max/m_star-1);
	
	return Rm

end

//CALCULATE THE CHARGE FRACTION

//macro test_charge_fraction()
//variable dp,phi,T
//dp = 930e-9
//phi =3
//T=300
//variable fn
// fn = eq_charge_frac(dp,phi,Temperature)
//print fn
//end

function eq_charge_frac(dp,phi,Temperature)
	variable dp,phi,Temperature
	//Weidensohler equations - from DMA manual
	//dp is in meters
	//T is in Kelvin
	variable e_const,epi,k_boltz,Z_Z
	variable i=0
	variable phi_index = phi-1
	e_const = 1.602177e-19
	epi=8.85418e-12;  //dielectric constant farad/m (for air)
	k_boltz=1.38065e-23 //Boltzmann's constant (for air)
	Z_Z=0.875; //ion mobility ratio (Wiedensohler)
	variable fn
	
	if (abs(phi)<3)
		make/O/N=(5,6) a
		make/O/N=(6) temp
		//Because you cannot assign rows to a matrix in IGOR, must put in as columns then transpose (to make things easier to input)
		a[][0] = {-26.3328, -2.3197, -0.0003, -2.3484 ,-44.4756}
		a[][1] = {35.9044, 0.6175, -0.1014, 0.6044, 79.3772}
		a[][2] = {-21.4608, 0.6201, 0.3073 ,0.4800, -62.8900}
		a[][3] = {7.0867, -0.1105, -0.3372, 0.0013 ,26.4492}
		a[][4] = { -1.3088, -0.1260, 0.1023, -0.1553,-5.7480}
		a[][5] = {0.1051, 0.0297, -0.0105, 0.0320, 0.5049}
		matrixOP/O coeff = a^t 

		for (i=0;i<6;i+=1)
			temp[i] = coeff[i][phi_index+3]*log(dp*1e9)^(i)
		endfor
		
		fn = 10^sum(temp)
		
		if (dp<20e-9 &abs(phi)==2)
			fn=0
		endif

	else
		fn = e_const/sqrt(4*pi*pi*epi*dp*k_boltz*Temperature)* exp(-(phi-2*pi*epi*dp*k_boltz*Temperature*ln(Z_Z)/e_const^2)^2/(4*pi*epi*dp*k_boltz*Temperature/e_const^2));
		//print fn
		if (dp<=69.78e-9 &abs(phi)>=3)
			fn=0
		endif
		if (fn<=6*10^-5)
			fn=0
		endif
	endif
	return fn
end

// Transfer function


//FUNCTION - SLIP CORRECTION
function cunningham_correction(d,P, T)
	//This function gives the cunningham slip correction
	//factor of particles with diamter d (m) at pressure P (atm) and T (K)
	//Cunningham slip correction parameters (Allen and Raabe, 1985)
	variable d, P, T
	variable alpha_cc, beta_cc, gamma_cc, la, lap, cun
	
	alpha_cc   = 1.142*2;
	beta_cc   = .558*2;
	gamma_cc   = .999/2;
	 
	la      = 67.3e-9;        // mfp of air at 101.325kPa and 296.15 K
	lap     = la*(T/296.15)*(1/P)*((1+110.4/296.15)/(1+110.4/T)) //mfp of air at new P and T
	 
	cun     = 1+((lap)/d)*(alpha_cc + beta_cc*exp(-gamma_cc*d/lap));
	
	return cun

end

//FUNCTION - DP2MOB
function dp2mobility(dp,Pressure,Temperature)
	//This function converts mobility to particle size.
	// dp = the particle mobility diameter (m)
	// P = the pressure in atm
	// T = the temperature in K
	variable dp, Pressure, Temperature
	variable mu, B
	mu = (1.81e-5)*(Temperature/293)^(0.74) //equation 2.29 in Hinds
	B = cunningham_correction(dp,Pressure,Temperature)/3/pi/mu/dp
	
	return B
end

//FUNCTION - mass2mob
function mass2mob(m,mass_mob_pref,mass_mob_exp,Pressure,Temperature)
	variable m, mass_mob_pref,mass_mob_exp,Pressure,Temperature
	
	variable d_m = (m/mass_mob_pref)^(1/mass_mob_exp)
	variable B = dp2mobility(d_m,Pressure,Temperature)
	return B
end


//// test Diffusion transfer function 
//macro test_diff_tf()
//	variable phi = 1
//	variable i = 5
//	variable m_test =  1.017090042735813e-18
//	variable B_test  =  7.125452944527521e+10
//	variable omega_test = 583
//	variable V_test = 1.211500000000000e2
//	variable Rm_test = 9.737269066115839
//	variable electric_charge = electric_charge_constant * phi
//	 flowrate_CPMA = 1.8667e-05
//	//print mass2mob(m_test,C,Dm,Pressure,Temperature)
//
//	print CPMA_diff_model_fixedgrid(m_test,B_test,electric_charge,flowrate_CPMA,omega_test,V_test,Temperature,omega_hat)
//end

//
//CPMA DIFFUSION TRANSFER FUNCTION//
//
function CPMA_diff_model_fixedgrid(m_star,B ,electric_charge,flowrate_CPMA,omega,V,Temperature,omega_hat)
	variable m_star, B, electric_charge, flowrate_CPMA, omega, V, Temperature,omega_hat
	variable CPMA_gap, CPMA_r1, CPMA_r2, CPMA_rhat, CPMA_omega_hat, CPMA_rc, CPMA_L, CPMA_k
	CPMA_gap   =   .001            //  gap - distance between cylinders (m)
	CPMA_r1   =   .060;              // r1 - inner radius of APM (m)
	CPMA_r2   =   CPMA_r1+CPMA_gap;             // r2 - outer radius of APM (m)
	CPMA_rhat   =   CPMA_r1/CPMA_r2;   //r_hat
	CPMA_rc   =   (CPMA_r2+CPMA_r1)/2;       //   rc - the center radius
	CPMA_L   =   .200;              // L - length of electrode (cylinder) (m)
	CPMA_k   =   1.3806503*10^-23;  // k - boltzmann's constant
	CPMA_omega_hat   =   omega_hat;
	
	variable omega1
	omega1= omega / ((CPMA_rhat^2-CPMA_omega_hat)/(CPMA_rhat^2-1)+CPMA_r1^2*(CPMA_omega_hat-1)/(CPMA_rhat^2-1)/CPMA_rc^2)
	
	variable r_size = 100 //number of r steps 
	variable D= 1.3806488e-23* Temperature*B //diffusion coefficient
	// - find the forces - //
	wave r = linspace(CPMA_r1, CPMA_r2,r_size)
	//calculate the velocity profile for the couette flow
	variable NU = CPMA_r1/CPMA_rc
	variable MU = omega/omega1
	make/O/N=(numpnts(r)) velocity= omega1 * (NU^2 - MU) / (NU^2 - 1) * r + omega1 * CPMA_r1^2 * (MU - 1) / (NU^2 - 1) / r
	make/O/N=(numpnts(r)) Fc = m_star * velocity^2 / r;       //%centrifugal force (N) (curve fit) //deal with q
	make/O/N=(numpnts(r)) Fe = electric_charge * V / r / ln(CPMA_r2 / CPMA_r1);    //%the electrostatic force (N)
	make/O/N=(numpnts(r)) delta_F = Fc-Fe;
	
	//transform cylinder geometery to flat plates dimension
	make/O/N=(numpnts(r)) y_var = r - CPMA_rc
	
	//calculate the coefficients of the polynomial which fits the force function across the gap
	make/O/D/N=3 a_coeff = 1
	CurveFit/Q=1/N=1/NTHR=0 poly 3, kwCWave=a_coeff,  delta_F /X=y_var /D 
	//CurveFit/NTHR=0 poly 3,  delta_F /X=y_var /D=a_coeff 
	//print a_coeff
	
	//calculate the flow rate
	variable A = pi*(CPMA_r2^2-CPMA_r1^2)
	variable v_flow = flowrate_CPMA/A //average flow velocity
	
	//calculate the non-dimensional variables - see Olfert first year PhD summary
	variable a2_tld = (CPMA_gap/2)^3 *B / D * a_coeff[2];
	variable a1_tld = (CPMA_gap/2)^2 *B / D * a_coeff[1];
	variable a0_tld = (CPMA_gap/2)   *B / D * a_coeff[0];
	variable x_max = 2/3 * D * CPMA_L / v_flow / (CPMA_gap/2)^2;
	
	//the initial number of y and x steps
	variable m = 50
	variable big_H = 2*m+1
	variable big_K = big_H
	//some constants
	variable tol = 1/100 //tolerance
	variable error  = 1 //initialize error
	variable j = 1 //initialize counter
	variable T_old = 1 
	variable loop_control = 0
	
	//used to be a loop
	//while error>tol
	
	variable small_h = 2/(big_H-1) //step size in y~
	variable small_k = x_max/(big_K-1) //step size in x~
	//starting vector (initial conditions)
	make/O/D/N=(big_H) n_old =1
	//wave wOut= linspace(1,1,big_H)
	//n_old[] = wOut[p]
	//print n_old
	//print n_old
	//need to test this and make sure it works.
	
	make/O/N=(big_H) vel_flow = 0
	variable y_temp = 0
	variable jj = 0
	
	//calculate a velocity vector for the parabolic flow
	for (jj=0;jj<big_H;jj+=1)
		y_temp = (jj)*small_h - 1
		vel_flow[jj] = 1-y_temp^2
	endfor
	
	make/O/N=(big_H) small_n = n_old
	variable count = 0
	
	//calculate the initial concentration of particles
	make/O/N=(big_H) xwave=p //scaling for integrate function called areaxy
	//print xwave
	make/O/N=(big_H) temp=n_old*vel_flow
	variable flux_in  = areaxy(xwave,temp)
	//print flux_in
	
	//variables for systems of equations
	variable x=0 //counting variable
	variable i=0
	variable W=0
	variable Q=0
	variable U=0
	variable flux_out
	variable T_new
	
	//matricies for system of equations
	make/O/N=(big_H-1) b_TD //middle of tridiagnol
	make/O/N=(big_H-2) a_TD //top of tridiag
	make/O/N=(big_H-2) c_TD //bottom of tridiag
	make/O/N=(big_H-1) RHS
	make/O/N=(big_H-1) n_new
	make/O/N=(big_H-1) temp2
	//TOMORROW - MAKE MATRICIES AND SOLVE!!!
	// to solve tri diag, convert to tri-diag using MatrixOP tridiag, then MatrixLinearSolveTD
	
	//print flux_in
	do 
	x +=small_k
	
	//print x
	//set up systems of equations to solve
		for (i=1;i<big_H-2;i+=1) //should that be the case?
			y_temp  = (i)*small_h-1
			W = 2*a2_tld*y_temp + a1_tld;
			Q = a2_tld*y_temp^2 + a1_tld*y_temp + a0_tld;
			U = 1 - y_temp^2;
			//print y_temp
			//print i
			
			b_TD[i] = U/small_k + 1/small_h^2 + W/2;
			a_TD[i-1] = -1/2/small_h^2 - Q/4/small_h;
	             c_TD[i] = Q/4/small_h - 1/2/small_h^2;
	             RHS[i] = n_old[i-1]*(1/2/small_h^2 + Q/4/small_h) + n_old[i]*(U/small_k - 1/small_h^2 - W/2) + n_old[i+1]*(1/2/small_h^2 - Q/4/small_h);	
		endfor
	
		//boundary conditions
		b_TD[0] = 1
		b_TD[big_H-1] = 1
		c_TD[0] = 0;
	       a_TD[big_H-2] = 0;
	       RHS[0] = 0;
	       RHS[big_H-1] = 0;
		//now solve it! - TOMMORROWS PROBLEM!!!
		MatrixOp/O  destwave = tridiag(c_TD,b_TD,a_TD)
		MatrixLinearSolveTD c_TD,b_TD,a_TD,RHS
		wave M_TDLinearSolution
		n_new = M_TDLinearSolution
		//print dimsize(destwave,0)
		//print dimsize(destwave,1)
		//print dimsize(RHS,0)
		 temp2 = n_new*vel_flow
		 flux_out = areaxy(xwave,temp2)
		 T_new = flux_out/flux_in
		 count +=1
		 
		 //loop control
		 if (T_new<0.0005)
		 	//break
		 	//print 2
		 endif 
		 
		 n_old = n_new
		 
	while (x<=x_max)
	T_old = T_new
	
	//print omega1
	//print D
	//print r
	//print NU
	//print MU
	//print v_flow
	
	
	return T_old
	


end




//Triangular transfer function - not tested yet 
function CPMA_triangle_tf(Rm,x_tilde)
	variable Rm, x_tilde 
	variable Omega = 0
	variable Beta_var
	
	Beta_var = 1/Rm
	Omega = 1/(2*Beta_var)*(abs(x_tilde-(1+Beta_var)) + abs(x_tilde-(1-Beta_var)) - 2*abs(x_tilde-1))
	
	return Omega
end



function/WAVE calcBinLimits(m_star,V,omega,addUpperBins)
	//This function calculates bin limits for the kernel function
	wave m_star,V,omega
	variable addUpperBins
	nvar max_charge, electric_charge_constant,mass_mob_pref,flowrate_CPMA,mass_mob_exp,Pressure,Temperature,omega_hat
	
	
	
	variable jj=0
	make/O/N=(numpnts(m_star)+1) bin_limits
	duplicate/O bin_limits binLimitsTest
	print numpnts(bin_limits)
	for (jj =0;jj<numpnts(m_star)-1;jj+=1)
		bin_limits[jj+1]=sqrt(m_star[jj]*m_star[jj+1])
	endfor
	
	bin_limits[0] = m_star[0]^2/bin_limits[1]
	bin_limits[numpnts(bin_limits)] = m_star[numpnts(m_star)]^2/bin_limits[numpnts(bin_limits)-2]
	binLimitsTest = bin_limits*1e18
//	print jj
	print binLimitsTest
	
	if (addUpperBins==1)
		
		//Now, in case multiple charging results in particles spilling over the highest bin
		// First, calculate the maximum m_star based on max number of charges estimated
		variable max_m_star =  calc_m_star(max_charge*electric_charge_constant,V[numpnts(V)],omega[numpnts(omega)]) //the highest mass occurs at charge state =max_charge
		// Now calculate the right edge of the diffusion CPMA transfer function
		variable m_max = calc_m_max(max_m_star,max_charge*electric_charge_constant,omega[numpnts(omega)],V[numpnts(V)],mass_mob_pref,flowrate_CPMA,mass_mob_exp,Pressure,Temperature,omega_hat)
		m_max*=1.1 //make it a bit higher because diffusion model is not perfect
		//print m_max
		//print bin_limits
		variable ii = numpnts(bin_limits)
		variable original_bin_limits = ii
		variable bins_added_on_right =0
		
			//Now add bins until they are bigger than m_max
			do 
				redimension/N=(numpnts(bin_limits)+1) bin_limits
				print bin_limits
				ii = numpnts(bin_limits)
				bin_limits[ii]=bin_limits[ii-2]^2/bin_limits[ii-3]
				
		//		print bin_limits[numpnts(bin_limits)-1]^2*1e18
		//		print bin_limits[numpnts(bin_limits)-2]*1e18
		//		print bin_limits
				bins_added_on_right= ii-original_bin_limits+1
				ii+=1
				print m_max
				print bin_limits[numpnts(bin_limits)]
			while (m_max>bin_limits[numpnts(bin_limits)-2]) //To make condition consistent with MATLAB
		
	endif
	
	//print bin_limits
	//Calculate the log constant if needed
	make/O/N=(numpnts(bin_limits)-1) log_constant
	for (jj=0;jj<numpnts(bin_limits)-1;jj+=1)
		log_constant[jj] = ln(bin_limits[jj+1]/bin_limits[jj])
	endfor
	variable bin_constant  = median2(log_constant) // Median log constant, because not exactly the same for each case
	//print bin_constant
	
	return bin_limits
	
end


macro testBinLimitsFCN()
	cereal_killer()
	load_constants() //load in the constants
	make/O/N=(21) m_star 
	make/O/N=(21) V
	make/O/N=(21) omega
	
	m_star = {0.302558851291677,	0.379028933613948,	0.474214577743928,0.597590235728896,	0.752204682925449,	0.942514017761424,	1.18243789652604,	1.48323714067915,	1.86590364523805,	2.33674657730179,	2.92633974704159,	3.67458328750748,	4.61189683047381,	5.78658336820895,	7.25778116723174,	9.12514302689620,	11.4831335514312,	14.4076212641928,	18.0007774565582,	22.4937965088858,	28.4530578377246}
	V = {62.47,71.49,81.72,93.31,106.4,121.15,137.8,156.46,177.37,200.71,226.79,255.76,287.95,323.59,362.97	,406.39,454.13,506.5,563.87	,626.65,695.45}
	omega = {739.400000000000,706.700000000000,675.500000000000,643,612,583.400000000000,555.500000000000,528.500000000000,501.700000000000,476.900000000000,453,429.300000000000,406.600000000000,384.800000000000,363.900000000000,343.400000000000,323.600000000000,305.100000000000,288,271.600000000000,254.400000000000}
	m_star*=1e-18
	
	
	calcBinLimits(m_star,V,omega)
	
end 



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Calculate N_SP2 AND dN/dlogm_rBC from SP2 data
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function calc_N_SP2(m_star, ref_timing, SP2_raw_data,Matrix,flowrate, V, omega)
	wave m_star, ref_timing, SP2_raw_data, Matrix, V, omega
	variable flowrate
	variable num_rows = dimsize(Matrix,0)
	make/O/N=(numpnts(m_star)) N_SP2=0
	// we have a sorted matrix (SP2_raw_data), but now we need to find all the SP2 data points for each CPMA scan in the file.
	// The ref_timing gives the start time of each CPMA scan
	//This means we need to make a new vector with the start and end of each scan in a loop, and search the SP2 data to find the points between each scan
	//there are 8 blank spaces below so ne
	make/O/N=(num_rows*2) m_star_step
	make/O/N=(num_rows*2) CPMA_scan_time
	make/O/N=(dimsize(SP2_raw_data,0)) BC_mass = SP2_raw_data[p][0]
	make/O/N =(dimsize(SP2_raw_data,0)) SP2_time = SP2_raw_data[p][1]
	
	//find the locations of the current CPMA scan in the SP2 file
	variable loc_SP2_data_end = round(binarysearchinterp(SP2_time,ref_timing[0]+30))
	//print loc_SP2_data_end
	variable loc_SP2_data_start = round(binarysearchinterp(SP2_time,ref_timing[num_rows]-30))
	//print loc_SP2_data_start
	
	// delete the points outside of the current CPMA scan to make things easier
	deletepoints/M=0 loc_SP2_data_end,numpnts(SP2_time)-loc_SP2_data_end, SP2_time
	deletepoints/M=0 0,loc_SP2_data_start, SP2_time
	deletepoints/M=0 loc_SP2_data_end,numpnts(BC_mass)-loc_SP2_data_end, BC_mass
	deletepoints/M=0 0,loc_SP2_data_start, BC_mass
	
	//counting variables
	variable ii = 0
	variable jj = 0
	// this loops creates bins for each CPMA setpoint (i.e. the "stair step" plot), starting 5 seconds after the file says it starts to account for the CPMA recording while the voltage is unstable. 
	//scan times were 30 seconds so take a 20 second "chunk"
	for (ii=0;ii<num_rows;ii+=1)
		//print ii
		m_star_step[2*ii] = m_star[ii]
		m_star_step[2*ii+1] = m_star[ii]
		CPMA_scan_time[2*ii] = ref_timing[ii]+25
		CPMA_scan_time[2*ii+1] = ref_timing[ii]+5
	endfor
	
	//make/O/N=(num_SP2_data) test1
	//Display m_star_step vs CPMA_scan_time; AppendToGraph BC_mass vs SP2_time
	Display BC_mass vs SP2_time; AppendToGraph m_star_step vs CPMA_scan_time
	ModifyGraph rgb(m_star_step)=(0,0,0),mode(BC_mass)=2
	
	//Now we need to count the number of particles in each CPMA setpoint and divide by time and flowrate
	//reset the counting variables 
	ii = 0
	jj = 0
	variable temp_N_SP2 =0
	variable temp_sum
	
	//calculate the bin edges using logrithmically spaced bins. Make the number of bins equal to the number of m_star's to make a square matrix
	variable bin_multiplier = 1
	//make/O/N=(numpnts(m_star)) set_edges

	//wave wOut = mylogspace(m_star[0],m_star[numpnts(m_star)],(numpnts(m_star)*bin_multiplier+1))
	//print wOut
	
	
	
	// Bin construction using a function (same bins as m_star)
	wave bin_limits = calcBinLimits(m_star,V,omega,0)
	duplicate bin_limits set_edges
	killwaves bin_limits
	
	//NEW SECTION HERE
	
//	// Need to find the Bin limits
//	make/O/N=(numpnts(m_star)+1) bin_limits
//	for (jj =1;jj<numpnts(m_star);jj+=1)
//		bin_limits[jj]=sqrt(m_star[jj]*m_star[jj+1])
//	endfor
//	bin_limits[0] = m_star[1]^2/m_star[2]
//	bin_limits[jj+2] = m_star[1]^2/m_star[2]
//	
//	nvar max_charge, electric_charge_constant,mass_mob_pref,flowrate_CPMA,mass_mob_exp,Pressure,Temperature,omega_hat
//	
//	variable max_m_star =  calc_m_star(max_charge*electric_charge_constant,V[numpnts(V)],omega[numpnts(omega)]) //the highest mass occurs at charge state =max_charge
//	variable m_max = calc_m_max(max_m_star,max_charge*electric_charge_constant,omega[numpnts(omega)],V[numpnts(V)],mass_mob_pref,flowrate_CPMA,mass_mob_exp,Pressure,Temperature,omega_hat)
//	m_max*=1.1 //make it a bit higher because diffusion model is not perfect
//	
//	i = numpnts(bin_limits)
//	variable original_bin_limits = i
//	variable bins_added_on_right =0
//	
//	do 
//		redimension/N=(numpnts(bin_limits)+1) bin_limits
//		bin_limits[i]=bin_limits[numpnts(bin_limits)-2]^2/bin_limits[numpnts(bin_limits)-3]
//		bins_added_on_right= i-original_bin_limits+1
//		i+=1
//	while (m_max>bin_limits[numpnts(bin_limits)])
//	
//	//need to make sure this works when particle diffusion is large and spills into the next bin
//		//this adds bin to the lower end if diffusion spills into the next bin
//		//first calcualte the difference between bin limits (logrithmic constant)
//	make/O/N=(numpnts(bin_limits)-1) log_constant
//	for (i=0;i<numpnts(bin_limits)-1;i+=1)
//		log_constant[i] = ln(bin_limits[i+1]/bin_limits[i])
//	endfor
//	variable bin_constant  = median2(log_constant)
	
	
	//NEW SECTION END
	
	
	//duplicate/O wOut set_edges
	//killwaves wOut
	make/O/N=(numpnts(set_edges)) m_upper
	make/O/N=(numpnts(set_edges)) m_lower
	//print set_edges
	m_upper = set_edges
	m_lower = set_edges
	deletepoints 0,1,m_upper
	deletepoints (numpnts(set_edges)-1),1,m_lower
	
	make/O/N=(numpnts(midpoints)) m_lower_midpoints
	m_lower_midpoints=set_edges[p]
	
	//begin loop to calcualte N_SP2 and dN/dlogm_rBC
	// for testing set, ii	
	//make a matrix for returning dN/dlogm_rBC
	make/O/N=(numpnts(set_edges)-1,numpnts(m_star)) waveOut
	//ii=10
	
	for (ii=0;ii<numpnts(m_star);ii+=1)
		//make/O/N=(dimsize(m_star,0))/FREE N_SP2
		
		//In the sorted SP2_times, find the beginning and end for each setpoint
		variable loc_setpoint_end = round(binarysearchinterp(SP2_time,CPMA_scan_time[2*ii]))
		//print loc_setpoint_end
		variable loc_setpoint_start = round(binarysearchinterp(SP2_time,CPMA_scan_time[2*ii+1]))
		//print loc_setpoint_start
		duplicate/O BC_mass, CPMA_setpoint_BC_mass //duplicate in order to perform operations without harming the data
		duplicate/O SP2_time, CPMA_setpoint_SP2_time
		
		//since the data is sorted, delete points to extract the BC mass data for each particle within the CPMA setpoint
		deletepoints loc_setpoint_end,(numpnts(CPMA_setpoint_BC_mass)-loc_setpoint_end), CPMA_setpoint_SP2_time
		deletepoints 0,loc_setpoint_start, CPMA_setpoint_SP2_time
		deletepoints loc_setpoint_end,numpnts(CPMA_setpoint_BC_mass)-loc_setpoint_end, CPMA_setpoint_BC_mass
		deletepoints 0,loc_setpoint_start, CPMA_setpoint_BC_mass
		
		//create a dummy variable
		make/O/N=(dimsize(CPMA_setpoint_BC_mass,0)) store_CPMA_setpoint_BC_mass = CPMA_setpoint_BC_mass
		
		//edit store_CPMA_setpoint_BC_mass
		//NOW: NEED TO GET RID OF NANS AND COUNT NUMBERS!!!
		variable kk = 0 //counting variable, need to loop through each value
		//for (ii=0;ii<dimsize() 
		
		//store_CPMA_setpoint_BC_mass = store_CPMA_setpoint_BC_mass[p] > 0 ? 1 : store_CPMA_setpoint_BC_mass[p];
		temp_N_SP2 =0
		temp_sum = 0
		
		//This loop finds the number of particles in each setpoint. eliminates 0's and nans by giving a range
			for (kk=0;kk<numpnts(store_CPMA_setpoint_BC_mass);kk+=1)
				if (1e10>store_CPMA_setpoint_BC_mass(kk)>0)
					temp_sum +=1
				endif 
			endfor
			
		//divide the #/setpoint by the time and flowrate to give #/cc 
		temp_N_SP2 = temp_sum/(abs(CPMA_scan_time[2*ii]-CPMA_scan_time[2*ii+1])*flowrate/60)
		N_SP2[ii] = temp_N_SP2 //put into a matrix to return
		
		//make a new wave to store the number/bin for making a histogram
		make/O/N=(numpnts(set_edges)-1) N_rBC=0
		
		//There is no command to make logrithmically spaced binned histogram. So this double loop does that
		for (jj=0;jj<numpnts(set_edges)-1;jj+=1)
			for (kk=0;kk<numpnts(CPMA_setpoint_BC_mass);kk+=1)
				if ((store_CPMA_setpoint_BC_mass[kk]>m_lower[jj]) & (store_CPMA_setpoint_BC_mass[kk]<m_upper[jj]))
					N_rBC[jj]+=1
				endif
			endfor
		endfor
		//edit store_CPMA_setpoint_BC_mass
		//edit m_lower
		//print N_rBC
		//This command takes the logrithmically spaced histogram wave (N_SP2) and divides by time and flowrate in order to give #/cc
		N_rBC/= (abs(CPMA_scan_time[2*ii]-CPMA_scan_time[2*ii+1])*flowrate/60)
		
		make/O/N=(numpnts(N_rBC)) dNdlogmrBC_temp=0
		//Calculate dN/dlogmp by dividing N_rBC by the log of the upper and lower bins. which is a constant
		dNdlogmrBC_temp = N_rBC/log(m_upper/m_lower)
		
		//Return a matrix with dNdlogmrBC in it
		for (kk=0;kk<numpnts(set_edges)-1;kk+=1)
			waveOut[kk][ii] = dNdlogmrBC_temp[kk]
		endfor
		
		if (ii==9)
			duplicate/O  N_rBC, test_wave
			duplicate/O dNdlogmrBC_temp, test_wave2
		endif
		
	endfor

	

	//here is a plact to test for equality between MATLAB and IGOR.
	display test_wave vs set_edges
	ModifyGraph mode=5,useNegRGB=1,usePlusRGB=1,hbFill=2,useNegPat=1,hBarNegFill=2;DelayUpdate
	ModifyGraph log(bottom)=1
	
	print N_SP2
	duplicate waveOut dNdlogmrBC_histograms //ie for histogarms
	duplicate dNdlogmrBC_temp dNdlogmrBC //as a function of m_Star
	killwaves dNdlogmrBC_temp, waveOut
	//killwaves CPMA_setpoint_BC_mass, CPMA_setpoint_SP2_time,store_CPMA_setpoint_BC_mass
end



